/**
  *  \file planetscentral/talk/message.h
  */
#ifndef C2SERVER_PLANETSCENTRAL_TALK_MESSAGE_H
#define C2SERVER_PLANETSCENTRAL_TALK_MESSAGE_H

#include "planetscentral/dbclient/dbclient.h"
#include "planetscentral/dbclient/dbsubtree.h"

class Topic;

class Message {
 public:
    Message(DbClient& client, int32_t messageId);

    DbHash            header();
    DbHashIntField    topicId();
    DbHashIntField    parentMessageId();
    DbHashIntField    postTime();
    DbHashIntField    editTime();
    DbHashStringField author();
    DbHashStringField subject();
    DbHashStringField rfcMessageId();
    DbHashStringField rfcHeaders();
    DbHashIntField    sequenceNumber();
    DbHashIntField    previousSequenceNumber();
    DbHashStringField previousRfcMessageId();

    DbString          text();

    bool              exists();
    void              remove();

    Topic             topic();

    int32_t           getId() const;
    IntValue*         describe();

    string_t          getRfcMessageId();
    string_t          getPreviousRfcMessageId();

    IntValue*         getRfcHeader();

    static int32_t    allocateMessage(DbClient& client);
    static int32_t    getLastId(DbClient& client);

    static void       removeRfcMessageId(DbClient& client, string_t id);
    static void       addRfcMessageId(DbClient& client, string_t id, int32_t messageId);
    static int32_t    lookupRfcMessageId(DbClient& client, string_t rfcMsgId);

 private:
    DbSubtree message;
    int32_t messageId;
};

#endif
