/**
  *  \file planetscentral/talk/syntaxc.h
  *  \brief Talk: Syntax Highlighter for C/C++/Java/JavaScript files
  */
#ifndef C2SERVER_PLANETSCENTRAL_TALK_SYNTAXC_H
#define C2SERVER_PLANETSCENTRAL_TALK_SYNTAXC_H

#include "planetscentral/talk/syntax.h"

/** Syntax highlighter for C/C++/Java/JavaScript files.
    These highlight C99, C++11, Java5, JavaScript5, with the following exceptions:
    - no trigraphs in C/C++ (e.g. "??/" would be a real backslash that could quote the quote ending a string)
    - no Unicode escapes in Java (e.g. "\u0022" is a real quote that terminates a string)
    - no raw string literals in C++
    - comments within preprocessor statements reset highlighting
    - backslash continuation is applied to all languages */
class SyntaxC : public SyntaxHighlighter {
 public:
    enum {
        LangC          = 1,
        LangCXX        = 2,
        LangJava       = 4,
        LangJavaScript = 8
    };

    SyntaxC(int language);
    ~SyntaxC();

    void init(const string_t& str);
    bool scan(Segment& result);

 private:
    int language;
    string_t str;
    string_t::size_type pos;

    /* States */
    enum State {
        sDefault,               /* Default state */
        sIncludeFileName,       /* After #include */
        sMacroName              /* After #define */
    };
    State state;

    /* Modifiers. These are always set according to the current position,
       but only evaluated when applicable to the current language. */
    bool accept_regexp;
    bool accept_preproc;
};

#endif
